﻿namespace etoy
{
    /// <summary>
    /// Meatdata信息
    /// </summary>
    class MetadataTable
    {
        /// <summary> Metadata表名 </summary>
        public string Name;
        /// <summary> 源文件 </summary>
        public string Path;
        /// <summary> 行数据 </summary>
        public MetadataRow[] Rows;

        // 结构体或者枚举 用于去重、数据校验、生成类型代码
        public Dictionary<string, MetadataStruct> Structs;
    }

    /// <summary>
    /// 行信息
    /// </summary>
    class MetadataRow
    {
        public string Type;
        public string Value;
        public string Description;
        public int Row;

        public MetadataStruct Struct;   // 由Value解析而来的
    }


    /// <summary>
    /// 自定义结构
    /// </summary>
    class MetadataStruct
    {
        public int Row;
        public string Name;
        public MetadataStructType StructType;
        public string Description;
        public MetadataField[] Fields;
    }

    enum MetadataStructType
    {
        /// <summary>
        /// 结构体
        /// </summary>
        Struct,
        /// <summary>
        /// 枚举
        /// </summary>
        Enum,
    }

    /// <summary>
    /// 自定义结构字段信息
    /// </summary>
    class MetadataField
    {
        public string Name;
        public string FieldType;
        public string EnumValue;
        /// <summary>
        /// 枚举的int值，配置表可能没填值，则从上一个有赋值的枚举开始递增
        /// </summary>
        public int EnumIntVal;
        public bool StructRepeated;
    }

    enum MetadataColumnType
    {
        Type,
        Value,
        Description
    }
}
